@@ -6,15 +6,19 @@ import re
 import dbstream
 import time
 import google.cloud.bigquery
+import pandas as pd
+from google.cloud import bigquery
+from google.cloud.bigquery import QueryJobConfig, LoadJobConfig, SchemaField
 from google.cloud.bigquery.dbapi import Cursor
 from googleauthentication import GoogleAuthentication
 
-from bigquery.core.Column import change_columns_type, columns_type_bool_to_str, change_column_value_to_string, \
-    change_column_value_to_int, change_column_value_to_float
+from bigquery.core.Column import change_columns_type, columns_type_bool_to_str, \
+    find_sample_value, detect_type, get_columns_type
 from bigquery.core.tools.print_colors import C
 from bigquery.core.Table import create_table, create_columns
 import logging
 
+
 class BigQueryDBStream(dbstream.DBStream):
     def __init__(self, instance_name, client_id, google_auth: GoogleAuthentication):
         super().__init__(instance_name, client_id=client_id)
@@ -65,64 +69,79 @@ class BigQueryDBStream(dbstream.DBStream):
             empty_list = []
             return empty_list
 
-    def _send(self, data, replace, batch_size=1000):
+    def _send(self, data, replace=True, batch_size=1000):
         print(C.WARNING + "Initiate send_to_bigquery on table " + data["table_name"] + "..." + C.ENDC)
-        client = self.connection()
-        con = google.cloud.bigquery.dbapi.connect(client=client)
-        cursor = Cursor(con)
+
         if replace:
-            cleaning_request = '''DELETE FROM ''' + data["table_name"] + ''' WHERE 1=1;'''
-            print(C.WARNING + "Cleaning table " + data["table_name"] + C.ENDC)
-            self.execute_query(cleaning_request)
-            print(C.OKGREEN + "[OK] Cleaning Done" + C.ENDC)
+            print(C.WARNING + "Table will be cleaned: " + data["table_name"] + C.ENDC)
 
-        boolean = True
-        index = 0
         total_rows = len(data["rows"])
-        total_nb_batches = len(data["rows"]) // batch_size + 1
-        while boolean:
-            temp_row = []
-            for i in range(batch_size):
-                if not data["rows"]:
-                    boolean = False
-                    continue
-                temp_row.append(data["rows"].pop())
 
-            final_data = []
-            for x in temp_row:
-                for y in x:
-                    if not isinstance(y, bool) \
-                            and not isinstance(y, datetime.date) \
-                            and not isinstance(y, datetime.datetime):
-                        try:
-                            y = int(y)
-                        except:
-                            try:
-                                y = float(y)
-                            except:
-                                pass
-                    final_data.append(y)
+        # Construct a BigQuery client object.
+        client = self.connection()
+        columns_name = data["columns_name"]
 
-            temp_string = ','.join(map(lambda a: '(' + ','.join(map(lambda b: '%s', a)) + ')', tuple(temp_row)))
+        df = pd.DataFrame(data["rows"], columns=columns_name)
+        file_path = "./%s.csv" % data["table_name"].replace('.', '_')
+        df.to_csv(file_path, index=False)
 
-            inserting_request = '''INSERT INTO ''' + data["table_name"] + ''' (''' + ", ".join(
-                data["columns_name"]) + ''') VALUES ''' + temp_string + ''';'''
-            if final_data:
-                try:
-                    cursor.execute(inserting_request, final_data)
-                except Exception as e:
-                    cursor.close()
-                    con.close()
-                    raise e
-            index = index + 1
-            percent = round(index * 100 / total_nb_batches, 2)
-            if percent < 100:
-                print("\r   %s / %s (%s %%)" % (str(index), total_nb_batches, str(percent)), end='\r')
+        params = {}
+        df = df.where((pd.notnull(df)), None)
+
+        table_name = data["table_name"].split('.')
+        columns_type = get_columns_type(self, table_name=table_name[1], schema_name=table_name[0])
+
+        for i in range(len(columns_name)):
+            name = columns_name[i]
+            col = dict()
+            if name in columns_type.keys():
+                col["type"] = columns_type[name]
+                params[name] = col
+                continue
+            example_max, example_min = find_sample_value(df, name, i)
+            col = dict()
+            col["example"] = example_max
+            type_max = detect_type(self, name=name, example=example_max)
+            if type_max == "TIMESTAMP" or type_max == "DATE":
+                type_min = detect_type(self, name=name, example=example_min)
+                if type_min == type_max:
+                    col["type"] = type_max
+                else:
+                    col["type"] = type_min
             else:
-                print("\r   %s / %s (%s %%)" % (str(index), total_nb_batches, str(percent)))
+                col["type"] = type_max
+            params[name] = col
+
+        schema = [
+            SchemaField(name=c, field_type=params[c]["type"]) for c in params.keys()
+        ]
+
+        job_config = bigquery.LoadJobConfig(
+            source_format=bigquery.SourceFormat.CSV,
+            skip_leading_rows=1,
+            schema=schema,
+            autodetect=True,
+            write_disposition="WRITE_TRUNCATE" if replace else "WRITE_APPEND"
+        )
+
+        table_id = os.environ["BIG_QUERY_PROJECT_ID"] + "." + data["table_name"]
+
+        with open(file_path, "rb") as source_file:
+            job = client.load_table_from_file(source_file, table_id, job_config=job_config)
+
+        job.result()  # Waits for the job to complete.
+
+        table = client.get_table(table_id)  # Make an API request.
+        print(
+            "Loaded {} rows and {} columns to {}".format(
+                table.num_rows, len(table.schema), table_id
+            )
+        )
 
         print(C.HEADER + str(total_rows) + ' rows sent to BigQuery table ' + data["table_name"] + C.ENDC)
         print(C.OKGREEN + "[OK] Sent to bigquery" + C.ENDC)
+        os.remove(file_path)
+        print(C.OKGREEN + "[OK] " + file_path + " deleted" + C.ENDC)
         return 0
 
     def _send_data_custom(self,
@@ -144,48 +163,21 @@ class BigQueryDBStream(dbstream.DBStream):
             self._send(data, replace=replace, batch_size=batch_size)
         except Exception as e:
             error_lowercase = str(e).lower()
-            logging.log(error_lowercase.split("\n")[0])
-            if ("value has type float64 which cannot be inserted into" in error_lowercase
-                or "value has type int64 which cannot be inserted into" in error_lowercase
-                or "value has type bool which cannot be inserted into" in error_lowercase) \
-                    and "string" in error_lowercase:
-                column = str(e).split("column ")[1].split(",")[0]
-                change_column_value_to_string(
+            if ("could not parse " in error_lowercase and "int64" in error_lowercase) \
+                    or ("could not parse " in error_lowercase and "double" in error_lowercase):
+                change_columns_type(
+                    self,
                     data=data_copy,
-                    column=column,
+                    other_table_to_update=other_table_to_update
                 )
-            elif (
-                    "value has type float64 which cannot be inserted into" in error_lowercase and not "string" in error_lowercase) \
-                    or (
-                    "value has type string which cannot be inserted into" in error_lowercase and not "bool" in error_lowercase):
-                column = str(e).split("column ")[1].split(",")[0]
-                if n == 2:
-                    change_columns_type(
-                        self,
-                        data=data_copy,
-                        other_table_to_update=other_table_to_update
-                    )
-                    n = 1
-                else:
-                    if "which has type int" in error_lowercase:
-                        change_column_value_to_int(
-                            data=data_copy,
-                            column=column,
-                        )
-                    elif "which has type float" in error_lowercase:
-                        change_column_value_to_float(
-                            data=data_copy,
-                            column=column,
-                        )
-                    n = 2
 
-            elif "value has type string which cannot be inserted into" in error_lowercase and "bool" in error_lowercase:
+            elif "could not parse " in error_lowercase and "bool" in error_lowercase:
                 columns_type_bool_to_str(
                     self,
                     data=data_copy,
                     other_table_to_update=other_table_to_update
                 )
-            elif " was not found " in error_lowercase and (
+            elif " not found:" in error_lowercase and (
                     " table " in error_lowercase or " dataset " in error_lowercase):
                 print("Destination table doesn't exist! Will be created")
                 create_table(
@@ -193,8 +185,7 @@ class BigQueryDBStream(dbstream.DBStream):
                     data=data_copy,
                     other_table_to_update=other_table_to_update
                 )
-                replace = False
-            elif " is not present in table " in error_lowercase and "column" in error_lowercase:
+            elif " provided schema does not match table " in error_lowercase and " cannot add fields " in error_lowercase:
                 create_columns(
                     self,
                     data=data_copy,
@@ -225,6 +216,8 @@ class BigQueryDBStream(dbstream.DBStream):
             r = self.execute_query("SELECT max(%s) as max FROM %s.%s %s" % (field, schema, table, filter_clause))
             return r[0]["max"]
         except Exception as e:
+            if "was not found" in str(e):
+                return None
             raise e
 
     def get_data_type(self, table_name, schema_name):
